//----------------------------------------------------------------------
// 2009.08.18
// Modified by Daniele Giacomini for `os16', to harmonize with it,
// even, when possible, on coding style.
//
// The original was taken form ELKS sources: `elkscmd/misc_utils/ed.c'.
//----------------------------------------------------------------------
//
// Copyright (c) 1993 by David I. Bell
// Permission is granted to use, distribute, or modify this source,
// provided that this copyright notice remains intact.
//
// The "ed" built-in command (much simplified)
//
//----------------------------------------------------------------------

#include <stdio.h>
#include <ctype.h>
#include <unistd.h>
#include <stdbool.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
//----------------------------------------------------------------------
#define isoctal(ch)  (((ch) >= '0') && ((ch) <= '7'))
#define USERSIZE     1024	/* max line length typed in by user */
#define INITBUFSIZE  1024	/* initial buffer size */
//----------------------------------------------------------------------
typedef int     num_t;
typedef int     len_t;
//
// The following is the type definition of structure `line_t', but the
// structure contains pointers to the same kind of type. With the
// compiler Bcc, it is the only way to declare it.
//
typedef struct line line_t;
//
struct line
{
  line_t         *next;
  line_t         *prev;
  len_t           len;
  char            data[1];
};
//
static line_t   lines;
static line_t  *curline;
static num_t    curnum;
static num_t    lastnum;
static num_t    marks[26];
static bool     dirty;
static char    *filename;
static char     searchstring[USERSIZE];
//
static char    *bufbase;
static char    *bufptr;
static len_t    bufused;
static len_t    bufsize;
//----------------------------------------------------------------------
static void     docommands (void);
static void     subcommand (char *cp, num_t num1, num_t num2);
static bool     getnum (char **retcp, bool * rethavenum,
			num_t * retnum);
static bool     setcurnum (num_t num);
static bool     initedit (void);
static void     termedit (void);
static void     addlines (num_t num);
static bool     insertline (num_t num, char *data, len_t len);
static bool     deletelines (num_t num1, num_t num2);
static bool     printlines (num_t num1, num_t num2, bool expandflag);
static bool     writelines (char *file, num_t num1, num_t num2);
static bool     readlines (char *file, num_t num);
static num_t    searchlines (char *str, num_t num1, num_t num2);
static len_t    findstring (line_t * lp, char *str, len_t len,
			    len_t offset);
static line_t  *findline (num_t num);
//----------------------------------------------------------------------
// Main.
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
  if (!initedit ())
    return (2);
  //
  if (argc > 1)
    {
      filename = strdup (argv[1]);
      if (filename == NULL)
	{
	  fprintf (stderr, "No memory\n");
	  termedit ();
	  return (1);
	}
      //
      if (!readlines (filename, 1))
	{
	  termedit ();
	  return (0);
	}
      //
      if (lastnum)
	setcurnum (1);
      //
      dirty = false;
    }
  //
  docommands ();
  //
  termedit ();
  return (0);
}

//----------------------------------------------------------------------
// Read commands until we are told to stop.
//----------------------------------------------------------------------
void
docommands (void)
{
  char           *cp;
  int             len;
  num_t           num1;
  num_t           num2;
  bool            have1;
  bool            have2;
  char            buf[USERSIZE];
  //
  while (true)
    {
      printf (": ");
      fflush (stdout);
      //
      if (fgets (buf, sizeof (buf), stdin) == NULL)
	{
	  return;
	}
      //
      len = strlen (buf);
      if (len == 0)
	{
	  return;
	}
      //
      cp = &buf[len - 1];
      if (*cp != '\n')
	{
	  fprintf (stderr, "Command line too long\n");
	  do
	    {
	      len = fgetc (stdin);
	    }
	  while ((len != EOF) && (len != '\n'));
	  //
	  continue;
	}
      //
      while ((cp > buf) && isblank (cp[-1]))
	{
	  cp--;
	}
      //
      *cp = '\0';
      //
      cp = buf;
      //
      while (isblank (*cp))
	{
	  //*cp++;
	  cp++;
	}
      //
      have1 = false;
      have2 = false;
      //
      if ((curnum == 0) && (lastnum > 0))
	{
	  curnum = 1;
	  curline = lines.next;
	}
      //
      if (!getnum (&cp, &have1, &num1))
	{
	  continue;
	}
      //
      while (isblank (*cp))
	{
	  cp++;
	}
      //
      if (*cp == ',')
	{
	  cp++;
	  if (!getnum (&cp, &have2, &num2))
	    {
	      continue;
	    }
	  //
	  if (!have1)
	    {
	      num1 = 1;
	    }
	  if (!have2)
	    {
	      num2 = lastnum;
	    }
	  have1 = true;
	  have2 = true;
	}
      //
      if (!have1)
	{
	  num1 = curnum;
	}
      if (!have2)
	{
	  num2 = num1;
	}
      //
      // Command interpretation switch.
      //
      switch (*cp++)
	{
	case 'a':
	  addlines (num1 + 1);
	  break;
	  //
	case 'c':
	  deletelines (num1, num2);
	  addlines (num1);
	  break;
	  //
	case 'd':
	  deletelines (num1, num2);
	  break;
	  //
	case 'f':
	  if (*cp && !isblank (*cp))
	    {
	      fprintf (stderr, "Bad file command\n");
	      break;
	    }
	  //
	  while (isblank (*cp))
	    {
	      cp++;
	    }
	  if (*cp == '\0')
	    {
	      if (filename)
		{
		  printf ("\"%s\"\n", filename);
		}
	      else
		{
		  printf ("No filename\n");
		}
	      break;
	    }
	  //
	  cp = strdup (cp);
	  //
	  if (cp == NULL)
	    {
	      fprintf (stderr, "No memory for filename\n");
	      break;
	    }
	  //
	  if (filename)
	    {
	      free (filename);
	    }
	  //
	  filename = cp;
	  break;
	  //
	case 'i':
	  addlines (num1);
	  break;
	  //
	case 'k':
	  while (isblank (*cp))
	    {
	      cp++;
	    }
	  //
	  if ((*cp < 'a') || (*cp > 'a') || cp[1])
	    {
	      fprintf (stderr, "Bad mark name\n");
	      break;
	    }
	  //
	  marks[*cp - 'a'] = num2;
	  break;
	  //
	case 'l':
	  printlines (num1, num2, true);
	  break;
	  //
	case 'p':
	  printlines (num1, num2, false);
	  break;
	  //
	case 'q':
	  while (isblank (*cp))
	    {
	      cp++;
	    }
	  //
	  if (have1 || *cp)
	    {
	      fprintf (stderr, "Bad quit command\n");
	      break;
	    }
	  //
	  if (!dirty)
	    {
	      return;
	    }
	  //
	  printf ("Really quit? ");
	  fflush (stdout);
	  //
	  buf[0] = '\0';
	  fgets (buf, sizeof (buf), stdin);
	  cp = buf;
	  //
	  while (isblank (*cp))
	    {
	      cp++;
	    }
	  //
	  if ((*cp == 'y') || (*cp == 'Y'))
	    {
	      return;
	    }
	  //
	  break;
	  //
	case 'r':
	  if (*cp && !isblank (*cp))
	    {
	      fprintf (stderr, "Bad read command\n");
	      break;
	    }
	  //
	  while (isblank (*cp))
	    {
	      cp++;
	    }
	  //
	  if (*cp == '\0')
	    {
	      fprintf (stderr, "No filename\n");
	      break;
	    }
	  //
	  if (!have1)
	    {
	      num1 = lastnum;
	    }
	  //
	  // Open the file and add to the buffer
	  // at the next line.
	  //
	  if (readlines (cp, num1 + 1))
	    {
	      //
	      // If the file open fails, just
	      // break the command.
	      //
	      break;
	    }
	  //
	  // Set the default file name, if no
	  // previous name is available.
	  //
	  if (filename == NULL)
	    {
	      filename = strdup (cp);
	    }
	  //
	  break;

	case 's':
	  subcommand (cp, num1, num2);
	  break;
	  //
	case 'w':
	  if (*cp && !isblank (*cp))
	    {
	      fprintf (stderr, "Bad write command\n");
	      break;
	    }
	  //
	  while (isblank (*cp))
	    {
	      cp++;
	    }
	  //
	  if (!have1)
	    {
	      num1 = 1;
	      num2 = lastnum;
	    }
	  //
	  // If the file name is not specified, use the
	  // default one.
	  //
	  if (*cp == '\0')
	    {
	      cp = filename;
	    }
	  //
	  // If even the default file name is not specified,
	  // tell it.
	  //
	  if (cp == NULL)
	    {
	      fprintf (stderr, "No file name specified\n");
	      break;
	    }
	  //
	  // Write the file.
	  //
	  writelines (cp, num1, num2);
	  //
	  break;
	  //
	case 'z':
	  switch (*cp)
	    {
	    case '-':
	      printlines (curnum - 21, curnum, false);
	      break;
	    case '.':
	      printlines (curnum - 11, curnum + 10, false);
	      break;
	    default:
	      printlines (curnum, curnum + 21, false);
	      break;
	    }
	  break;
	  //
	case '.':
	  if (have1)
	    {
	      fprintf (stderr, "No arguments allowed\n");
	      break;
	    }
	  printlines (curnum, curnum, false);
	  break;
	  //
	case '-':
	  if (setcurnum (curnum - 1))
	    {
	      printlines (curnum, curnum, false);
	    }
	  break;
	  //
	case '=':
	  printf ("%d\n", num1);
	  break;
	  //
	case '\0':
	  if (have1)
	    {
	      printlines (num2, num2, false);
	      break;
	    }
	  //
	  if (setcurnum (curnum + 1))
	    {
	      printlines (curnum, curnum, false);
	    }
	  break;
	  //
	default:
	  fprintf (stderr, "Unimplemented command\n");
	  break;
	}
    }
}

//----------------------------------------------------------------------
// Do the substitute command.
// The current line is set to the last substitution done.
//----------------------------------------------------------------------
void
subcommand (char *cp, num_t num1, num_t num2)
{
  int             delim;
  char           *oldstr;
  char           *newstr;
  len_t           oldlen;
  len_t           newlen;
  len_t           deltalen;
  len_t           offset;
  line_t         *lp;
  line_t         *nlp;
  bool            globalflag;
  bool            printflag;
  bool            didsub;
  bool            needprint;

  if ((num1 < 1) || (num2 > lastnum) || (num1 > num2))
    {
      fprintf (stderr, "Bad line range for substitute\n");
      return;
    }
  //
  globalflag = false;
  printflag = false;
  didsub = false;
  needprint = false;
  //
  if (isblank (*cp) || (*cp == '\0'))
    {
      fprintf (stderr, "Bad delimiter for substitute\n");
      return;
    }
  //
  delim = *cp++;
  oldstr = cp;
  //
  cp = strchr (cp, delim);
  //
  if (cp == NULL)
    {
      fprintf (stderr, "Missing 2nd delimiter for substitute\n");
      return;
    }
  //
  *cp++ = '\0';
  //
  newstr = cp;
  cp = strchr (cp, delim);
  //
  if (cp)
    {
      *cp++ = '\0';
    }
  else
    {
      cp = "";
    }
  while (*cp)
    {
      switch (*cp++)
	{
	case 'g':
	  globalflag = true;
	  break;
	  //
	case 'p':
	  printflag = true;
	  break;
	  //
	default:
	  fprintf (stderr, "Unknown option for substitute\n");
	  return;
	}
    }
  //
  if (*oldstr == '\0')
    {
      if (searchstring[0] == '\0')
	{
	  fprintf (stderr, "No previous search string\n");
	  return;
	}
      oldstr = searchstring;
    }
  //
  if (oldstr != searchstring)
    {
      strcpy (searchstring, oldstr);
    }
  //
  lp = findline (num1);
  if (lp == NULL)
    {
      return;
    }
  //
  oldlen = strlen (oldstr);
  newlen = strlen (newstr);
  deltalen = newlen - oldlen;
  offset = 0;
  //
  while (num1 <= num2)
    {
      offset = findstring (lp, oldstr, oldlen, offset);
      if (offset < 0)
	{
	  if (needprint)
	    {
	      printlines (num1, num1, false);
	      needprint = false;
	    }
	  //
	  offset = 0;
	  lp = lp->next;
	  num1++;
	  continue;
	}
      //
      needprint = printflag;
      didsub = true;
      dirty = true;

      //--------------------------------------------------------------
      // If the replacement string is the same size or shorter
      // than the old string, then the substitution is easy.
      //--------------------------------------------------------------

      if (deltalen <= 0)
	{
	  memcpy (&lp->data[offset], newstr, newlen);
	  //
	  if (deltalen)
	    {
	      memcpy (&lp->data[offset + newlen],
		      &lp->data[offset + oldlen],
		      lp->len - offset - oldlen);
	      //
	      lp->len += deltalen;
	    }
	  //
	  offset += newlen;
	  //
	  if (globalflag)
	    {
	      continue;
	    }
	  //
	  if (needprint)
	    {
	      printlines (num1, num1, false);
	      needprint = false;
	    }
	  //
	  lp = nlp->next;
	  num1++;
	  continue;
	}

      //--------------------------------------------------------------
      // The new string is larger, so allocate a new line
      // structure and use that.  Link it in in place of
      // the old line structure.
      //--------------------------------------------------------------

      nlp = (line_t *) malloc (sizeof (line_t) + lp->len + deltalen);
      //
      if (nlp == NULL)
	{
	  fprintf (stderr, "Cannot get memory for line\n");
	  return;
	}
      //
      nlp->len = lp->len + deltalen;
      //
      memcpy (nlp->data, lp->data, offset);
      //
      memcpy (&nlp->data[offset], newstr, newlen);
      //
      memcpy (&nlp->data[offset + newlen],
	      &lp->data[offset + oldlen], lp->len - offset - oldlen);
      //
      nlp->next = lp->next;
      nlp->prev = lp->prev;
      nlp->prev->next = nlp;
      nlp->next->prev = nlp;
      //
      if (curline == lp)
	{
	  curline = nlp;
	}
      //
      free (lp);
      lp = nlp;
      //
      offset += newlen;
      //
      if (globalflag)
	{
	  continue;
	}
      //
      if (needprint)
	{
	  printlines (num1, num1, false);
	  needprint = false;
	}
      //
      lp = lp->next;
      num1++;
    }
  //
  if (!didsub)
    {
      fprintf (stderr, "No substitutions found for \"%s\"\n", oldstr);
    }
}

//----------------------------------------------------------------------
// Search a line for the specified string starting at the specified
// offset in the line.  Returns the offset of the found string, or -1.
//----------------------------------------------------------------------
len_t
findstring (line_t * lp, char *str, len_t len, len_t offset)
{
  len_t           left;
  char           *cp;
  char           *ncp;
  //
  cp = &lp->data[offset];
  left = lp->len - offset;
  //
  while (left >= len)
    {
      ncp = memchr (cp, *str, left);
      if (ncp == NULL)
	{
	  return (len_t) - 1;
	}
      //
      left -= (ncp - cp);
      if (left < len)
	{
	  return (len_t) - 1;
	}
      //
      cp = ncp;
      if (memcmp (cp, str, len) == 0)
	{
	  return (len_t) (cp - lp->data);
	}
      //
      cp++;
      left--;
    }
  //
  return (len_t) - 1;
}

//----------------------------------------------------------------------
// Add lines which are typed in by the user.
// The lines are inserted just before the specified line number.
// The lines are terminated by a line containing a single dot (ugly!),
// or by an end of file.
//----------------------------------------------------------------------
void
addlines (num_t num)
{
  int             len;
  char            buf[USERSIZE + 1];
  //
  while (fgets (buf, sizeof (buf), stdin))
    {
      if ((buf[0] == '.') && (buf[1] == '\n') && (buf[2] == '\0'))
	{
	  return;
	}
      //
      len = strlen (buf);
      //
      if (len == 0)
	{
	  return;
	}
      //
      if (buf[len - 1] != '\n')
	{
	  fprintf (stderr, "Line too long\n");
	  //
	  do
	    {
	      len = fgetc (stdin);
	    }
	  while ((len != EOF) && (len != '\n'));
	  //
	  return;
	}
      //
      if (!insertline (num++, buf, len))
	{
	  return;
	}
    }
}

//----------------------------------------------------------------------
// Parse a line number argument if it is present.  This is a sum
// or difference of numbers, '.', '$', 'x, or a search string.
// Returns true if successful (whether or not there was a number). 
// Returns false if there was a parsing error, with a message output.
// Whether there was a number is returned indirectly, as is the number.
// The character pointer which stopped the scan is also returned.
//----------------------------------------------------------------------
static          bool
getnum (char **retcp, bool * rethavenum, num_t * retnum)
{
  char           *cp;
  char           *str;
  bool            havenum;
  num_t           value;
  num_t           num;
  num_t           sign;
  //
  cp = *retcp;
  havenum = false;
  value = 0;
  sign = 1;
  //
  while (true)
    {
      while (isblank (*cp))
	{
	  cp++;
	}
      //
      switch (*cp)
	{
	case '.':
	  havenum = true;
	  num = curnum;
	  cp++;
	  break;
	  //
	case '$':
	  havenum = true;
	  num = lastnum;
	  cp++;
	  break;
	  //
	case '\'':
	  cp++;
	  if ((*cp < 'a') || (*cp > 'z'))
	    {
	      fprintf (stderr, "Bad mark name\n");
	      return false;
	    }
	  //
	  havenum = true;
	  num = marks[*cp++ - 'a'];
	  break;
	  //
	case '/':
	  str = ++cp;
	  cp = strchr (str, '/');
	  if (cp)
	    {
	      *cp++ = '\0';
	    }
	  else
	    {
	      cp = "";
	    }
	  num = searchlines (str, curnum, lastnum);
	  if (num == 0)
	    {
	      return false;
	    }
	  //
	  havenum = true;
	  break;
	  //
	default:
	  if (!isdigit (*cp))
	    {
	      *retcp = cp;
	      *rethavenum = havenum;
	      *retnum = value;
	      return true;
	    }
	  //
	  num = 0;
	  while (isdigit (*cp))
	    {
	      num = num * 10 + *cp++ - '0';
	    }
	  havenum = true;
	  break;
	}
      //
      value += num * sign;
      //
      while (isblank (*cp))
	{
	  cp++;
	}
      //
      switch (*cp)
	{
	case '-':
	  sign = -1;
	  cp++;
	  break;
	  //
	case '+':
	  sign = 1;
	  cp++;
	  break;
	  //
	default:
	  *retcp = cp;
	  *rethavenum = havenum;
	  *retnum = value;
	  return true;
	}
    }
}

//----------------------------------------------------------------------
// Initialize everything for editing.
//----------------------------------------------------------------------
bool
initedit (void)
{
  int             i;
  //
  bufsize = INITBUFSIZE;
  bufbase = malloc (bufsize);
  //
  if (bufbase == NULL)
    {
      fprintf (stderr, "No memory for buffer\n");
      return false;
    }
  //
  bufptr = bufbase;
  bufused = 0;
  //
  lines.next = &lines;
  lines.prev = &lines;
  //
  curline = NULL;
  curnum = 0;
  lastnum = 0;
  dirty = false;
  filename = NULL;
  searchstring[0] = '\0';
  //
  for (i = 0; i < 26; i++)
    {
      marks[i] = 0;
    }
  //
  return true;
}

//----------------------------------------------------------------------
// Finish editing.
//----------------------------------------------------------------------
void
termedit (void)
{
  if (bufbase)
    free (bufbase);
  bufbase = NULL;
  //
  bufptr = NULL;
  bufsize = 0;
  bufused = 0;
  //
  if (filename)
    free (filename);
  filename = NULL;
  //
  searchstring[0] = '\0';
  //
  if (lastnum)
    deletelines (1, lastnum);
  //
  lastnum = 0;
  curnum = 0;
  curline = NULL;
}

//----------------------------------------------------------------------
// Read lines from a file at the specified line number.
// Returns true if the file was successfully read.
//----------------------------------------------------------------------
bool
readlines (char *file, num_t num)
{
  int             fd;
  int             cc;
  len_t           len;
  len_t           linecount;
  len_t           charcount;
  char           *cp;
  //
  if ((num < 1) || (num > lastnum + 1))
    {
      fprintf (stderr, "Bad line for read\n");
      return false;
    }
  //
  fd = open (file, O_RDONLY);
  if (fd < 0)
    {
      perror (file);
      return false;
    }
  //
  bufptr = bufbase;
  bufused = 0;
  linecount = 0;
  charcount = 0;
  //
  printf ("\"%s\", ", file);
  fflush (stdout);
  //
  do
    {
      cp = memchr (bufptr, '\n', bufused);
      if (cp)
	{
	  len = (cp - bufptr) + 1;
	  //
	  if (!insertline (num, bufptr, len))
	    {
	      close (fd);
	      return false;
	    }
	  //
	  bufptr += len;
	  bufused -= len;
	  charcount += len;
	  linecount++;
	  num++;
	  continue;
	}
      //
      if (bufptr != bufbase)
	{
	  memcpy (bufbase, bufptr, bufused);
	  bufptr = bufbase + bufused;
	}
      //
      if (bufused >= bufsize)
	{
	  len = (bufsize * 3) / 2;
	  cp = realloc (bufbase, len);
	  if (cp == NULL)
	    {
	      fprintf (stderr, "No memory for buffer\n");
	      close (fd);
	      return false;
	    }
	  //
	  bufbase = cp;
	  bufptr = bufbase + bufused;
	  bufsize = len;
	}
      //
      cc = read (fd, bufptr, bufsize - bufused);
      bufused += cc;
      bufptr = bufbase;
    }
  while (cc > 0);
  //
  if (cc < 0)
    {
      perror (file);
      close (fd);
      return false;
    }
  //
  if (bufused)
    {
      if (!insertline (num, bufptr, bufused))
	{
	  close (fd);
	  return -1;
	}
      linecount++;
      charcount += bufused;
    }
  //
  close (fd);
  //
  printf ("%d lines%s, %d chars\n",
	  linecount, (bufused ? " (incomplete)" : ""), charcount);
  //
  return true;
}

//----------------------------------------------------------------------
// Write the specified lines out to the specified file.
// Returns true if successful, or false on an error with a message
// output.
//----------------------------------------------------------------------
bool
writelines (char *file, num_t num1, num_t num2)
{
  int             fd;
  line_t         *lp;
  len_t           linecount;
  len_t           charcount;
  //
  if ((num1 < 1) || (num2 > lastnum) || (num1 > num2))
    {
      fprintf (stderr, "Bad line range for write\n");
      return false;
    }
  //
  linecount = 0;
  charcount = 0;
  //
  fd = creat (file, 0666);
  if (fd < 0)
    {
      perror (file);
      return false;
    }
  //
  printf ("\"%s\", ", file);
  fflush (stdout);
  //
  lp = findline (num1);
  if (lp == NULL)
    {
      close (fd);
      return false;
    }
  //
  while (num1++ <= num2)
    {
      if (write (fd, lp->data, lp->len) != lp->len)
	{
	  perror (file);
	  close (fd);
	  return false;
	}
      //
      charcount += lp->len;
      linecount++;
      lp = lp->next;
    }
  //
  if (close (fd) < 0)
    {
      perror (file);
      return false;
    }
  //
  printf ("%d lines, %d chars\n", linecount, charcount);
  //
  return true;
}

//----------------------------------------------------------------------
// Print lines in a specified range.
// The last line printed becomes the current line.
// If expandflag is true, then the line is printed specially to
// show magic characters.
//----------------------------------------------------------------------
bool
printlines (num_t num1, num_t num2, bool expandflag)
{
  line_t         *lp;
  unsigned char  *cp;
  int             ch;
  len_t           count;
  //
  if ((num1 < 1) || (num2 > lastnum) || (num1 > num2))
    {
      fprintf (stderr, "Bad line range for print\n");
      return false;
    }
  //
  lp = findline (num1);
  if (lp == NULL)
    {
      return false;
    }
  //
  while (num1 <= num2)
    {
      if (!expandflag)
	{
	  write (STDOUT_FILENO, lp->data, lp->len);
	  setcurnum (num1++);
	  lp = lp->next;
	  continue;
	}

      //--------------------------------------------------------------
      // Show control characters and characters with the
      // high bit set specially.
      //--------------------------------------------------------------

      cp = (unsigned char *) lp->data;
      count = lp->len;
      //
      if ((count > 0) && (cp[count - 1] == '\n'))
	{
	  count--;
	}
      //
      while (count-- > 0)
	{
	  ch = *cp++;
	  if (ch & 0x80)
	    {
	      fputs ("M-", stdout);
	      ch &= 0x7f;
	    }
	  if (ch < ' ')
	    {
	      fputc ('^', stdout);
	      ch += '@';
	    }
	  if (ch == 0x7f)
	    {
	      fputc ('^', stdout);
	      ch = '?';
	    }
	  fputc (ch, stdout);
	}
      //
      fputs ("$\n", stdout);
      //
      setcurnum (num1++);
      lp = lp->next;
    }
  //
  return true;
}

//----------------------------------------------------------------------
// Insert a new line with the specified text.
// The line is inserted so as to become the specified line,
// thus pushing any existing and further lines down one.
// The inserted line is also set to become the current line.
// Returns true if successful.
//----------------------------------------------------------------------
bool
insertline (num_t num, char *data, len_t len)
{
  line_t         *newlp;
  line_t         *lp;
  //
  if ((num < 1) || (num > lastnum + 1))
    {
      fprintf (stderr, "Inserting at bad line number\n");
      return false;
    }
  //
  newlp = (line_t *) malloc (sizeof (line_t) + len - 1);
  if (newlp == NULL)
    {
      fprintf (stderr, "Failed to allocate memory for line\n");
      return false;
    }
  //
  memcpy (newlp->data, data, len);
  newlp->len = len;
  //
  if (num > lastnum)
    {
      lp = &lines;
    }
  else
    {
      lp = findline (num);
      if (lp == NULL)
	{
	  free ((char *) newlp);
	  return false;
	}
    }
  //
  newlp->next = lp;
  newlp->prev = lp->prev;
  lp->prev->next = newlp;
  lp->prev = newlp;
  //
  lastnum++;
  dirty = true;
  //
  return setcurnum (num);
}

//----------------------------------------------------------------------
// Delete lines from the given range.
//----------------------------------------------------------------------
bool
deletelines (num_t num1, num_t num2)
{
  line_t         *lp;
  line_t         *nlp;
  line_t         *plp;
  num_t           count;
  //
  if ((num1 < 1) || (num2 > lastnum) || (num1 > num2))
    {
      fprintf (stderr, "Bad line numbers for delete\n");
      return false;
    }
  //
  lp = findline (num1);
  if (lp == NULL)
    {
      return false;
    }
  //
  if ((curnum >= num1) && (curnum <= num2))
    {
      if (num2 < lastnum)
	{
	  setcurnum (num2 + 1);
	}
      else if (num1 > 1)
	{
	  setcurnum (num1 - 1);
	}
      else
	{
	  curnum = 0;
	}
    }
  //
  count = num2 - num1 + 1;
  //
  if (curnum > num2)
    {
      curnum -= count;
    }
  //
  lastnum -= count;
  //
  while (count-- > 0)
    {
      nlp = lp->next;
      plp = lp->prev;
      plp->next = nlp;
      nlp->prev = plp;
      lp->next = NULL;
      lp->prev = NULL;
      lp->len = 0;
      free (lp);
      lp = nlp;
    }
  //
  dirty = true;
  //
  return true;
}

//----------------------------------------------------------------------
// Search for a line which contains the specified string.
// If the string is NULL, then the previously searched for string
// is used.  The currently searched for string is saved for future use.
// Returns the line number which matches, or 0 if there was no match
// with an error printed.
//----------------------------------------------------------------------
num_t
searchlines (char *str, num_t num1, num_t num2)
{
  line_t         *lp;
  int             len;
  //
  if ((num1 < 1) || (num2 > lastnum) || (num1 > num2))
    {
      fprintf (stderr, "Bad line numbers for search\n");
      return 0;
    }
  //
  if (*str == '\0')
    {
      if (searchstring[0] == '\0')
	{
	  fprintf (stderr, "No previous search string\n");
	  return 0;
	}
      str = searchstring;
    }
  //
  if (str != searchstring)
    {
      strcpy (searchstring, str);
    }
  //
  len = strlen (str);
  //
  lp = findline (num1);
  if (lp == NULL)
    {
      return 0;
    }
  //
  while (num1 <= num2)
    {
      if (findstring (lp, str, len, 0) >= 0)
	{
	  return num1;
	}
      //
      num1++;
      lp = lp->next;
    }
  //
  fprintf (stderr, "Cannot find string \"%s\"\n", str);
  //
  return 0;
}

//----------------------------------------------------------------------
// Return a pointer to the specified line number.
//----------------------------------------------------------------------
line_t         *
findline (num_t num)
{
  line_t         *lp;
  num_t           lnum;
  //
  if ((num < 1) || (num > lastnum))
    {
      fprintf (stderr, "Line number %d does not exist\n", num);
      return NULL;
    }
  //
  if (curnum <= 0)
    {
      curnum = 1;
      curline = lines.next;
    }
  //
  if (num == curnum)
    {
      return curline;
    }
  //
  lp = curline;
  lnum = curnum;
  //
  if (num < (curnum / 2))
    {
      lp = lines.next;
      lnum = 1;
    }
  else if (num > ((curnum + lastnum) / 2))
    {
      lp = lines.prev;
      lnum = lastnum;
    }
  //
  while (lnum < num)
    {
      lp = lp->next;
      lnum++;
    }
  //
  while (lnum > num)
    {
      lp = lp->prev;
      lnum--;
    }
  //
  return lp;
}

//----------------------------------------------------------------------
// Set the current line number.
// Returns true if successful.
//----------------------------------------------------------------------
bool
setcurnum (num_t num)
{
  line_t         *lp;
  //
  lp = findline (num);
  if (lp == NULL)
    {
      return false;
    }
  //
  curnum = num;
  curline = lp;
  //
  return true;
}

/* END CODE */
