#include <unistd.h>
#include <errno.h>
//----------------------------------------------------------------------
#define MAX_LINES   20
#define MAX_COLUMNS 80
//----------------------------------------------------------------------
static void     usage (void);
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
  FILE           *fp;
  char           *name;
  int             c;
  int             line = 1;	// Line internal counter.
  int             column = 1;	// Column internal counter.
  int             a;		// Index inside arguments.
  int             loop;
  //
  // There must be at least an argument, plus the program name.
  //
  if (argc < 2)
    {
      usage ();
      return (1);
    }
  //
  // No options are allowed.
  //
  for (a = 1; a < argc; a++)
    {
      //
      // Get next name from arguments.
      //
      name = argv[a];
      //
      // Try to open the file, read only.
      //
      fp = fopen (name, "r");
      //
      if (fp == NULL)
	{
	  //
	  // Error opening file.
	  //
	  perror (name);
	  return (1);
	}
      //
      // Print the file name to be displayed.
      //
      printf ("== %s ==\n", name);
      line++;
      //
      // The following loop continues while the file
      // gives characters, or when a command to change
      // file or to quit is given.
      //
      for (loop = 1; loop;)
	{
	  //
	  // Read a single character.
	  //
	  c = getc (fp);
	  //
	  if (c == EOF)
	    {
	      loop = 0;
	      break;
	    }
	  //
	  // If the character read is a special one,
	  // the line/column calculation is modified,
	  // so that it is known when to stop scrolling.
	  //
	  switch (c)
	    {
	    case '\r':
	      //
	      // Displaying this character, the cursor should go
	      // back to the first column. So the column counter
	      // is reset.
	      //
	      column = 1;
	      break;
	    case '\n':
	      //
	      // Displaying this character, the cursor should go
	      // back to the next line, at the first column.
	      // So the column counter is reset and the line
	      // counter is incremented.
	      //
	      line++;
	      column = 1;
	      break;
	    case '\b':
	      //
	      // Displaying this character, the cursor should go
	      // back one position, unless it is already at the
	      // beginning.
	      //
	      if (column > 1)
		{
		  column--;
		}
	      break;
	    default:
	      //
	      // Any other character must increase the column
	      // counter.
	      //
	      column++;
	    }
	  //
	  // Display the character, even if it is a special one:
	  // it is responsibility of the screen device management
	  // to do something good with special characters.
	  //
	  putchar (c);
	  //
	  // If the column counter is gone beyond the screen columns,
	  // then adjust the column counter and increment the line
	  // counter.
	  //
	  if (column > MAX_COLUMNS)
	    {
	      column -= MAX_COLUMNS;
	      line++;
	    }
	  //
	  // Check if there is space for scrolling.
	  //
	  if (line < MAX_LINES)
	    {
	      continue;
	    }
	  //
	  // Here, displayed lines are MAX_LINES.
	  //
	  if (column > 1)
	    {
	      //
	      // Something was printed at the current line: must
	      // do a new line.
	      //
	      putchar ('\n');
	    }
	  //
	  // Show the more prompt.
	  //
	  printf ("--More--");
	  fflush (stdout);
	  //
	  // Read a character from standard input.
	  //
	  c = getchar ();
	  //
	  // Consider commands `n' and `q', but any other character
	  // can be introduced, to let show the next page.
	  //
	  switch (c)
	    {
	    case 'N':
	    case 'n':
	      //
	      // Go to the next file, if any.
	      //
	      fclose (fp);
	      fp = NULL;
	      loop = 0;
	      break;
	    case 'Q':
	    case 'q':
	      //
	      // Quit. But must erase the `--More--' prompt.
	      //
	      printf ("\b \b\b \b\b \b\b \b\b \b");
	      printf ("\b \b\b \b\b \b\b \b");
	      fclose (fp);
	      return (0);
	    }
	  //
	  // Backspace to overwrite `--More--' and the character
	  // pressed.
	  //
	  printf ("\b \b\b \b\b \b\b \b\b \b\b \b\b \b\b \b\b \b");
	  //
	  // Reset line/column counters.
	  //
	  column = 1;
	  line = 1;
	}
      //
      // Close the file pointer if it is still open.
      //
      if (fp != NULL)
	{
	  fclose (fp);
	}
    }
  //
  return (0);
}

//----------------------------------------------------------------------
static void
usage (void)
{
  fprintf (stderr, "Usage: more FILE...\n");
}
