#include <stdio.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdint.h>
#include <errno.h>
//----------------------------------------------------------------------
typedef struct {
  uint8_t   ihl      : 4,
            version  : 4;
  uint8_t   service;
  uint16_t  length;
  uint16_t  id;
  uint16_t  fragment;    // [flg][fragment_offset] 3+13 bit
  uint8_t   ttl;
  uint8_t   protocol;
  uint16_t  checksum;
  in_addr_t src;         // Network byte order.
  in_addr_t dst;         // Network byte order.
} __attribute__ ((packed)) xip_t;

typedef struct {
  uint8_t  type;
  uint8_t  code;
  uint16_t checksum;
} __attribute__ ((packed)) xicmp_t;

typedef struct {
  uint16_t identifier;
  uint16_t sequence;
} __attribute__ ((packed)) xecho_t;

typedef struct {
  xip_t   ip;
  xicmp_t icmp;
  xecho_t echo;
  char data[60];
} __attribute__ ((packed)) xip_packet_t;

typedef struct {
  xicmp_t icmp;
  xecho_t echo;
  char data[60];
} __attribute__ ((packed)) xicmp_packet_t;
//----------------------------------------------------------------------
static uint16_t ip_chk (uint16_t *data, size_t size);
static void     usage (void);
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
  int                sfdn;
  struct sockaddr_in sa;
  ssize_t            sent;
  ssize_t            received;
  int                status;
  char              *destination;
  uint16_t           checksum;
  xicmp_packet_t     icmp_packet_send;
  xip_packet_t       ip_packet_receive;
  int                retry = 3;         // Max send retry.
  //
  // No options are known, but an argument must be given.
  //
  if (argc < 2)
    {
      usage ();
      return (1);
    }
  destination = argv[1];
  //
  // Define the destination `sa'
  //
  sa.sin_family = AF_INET;
  sa.sin_port=0;
  inet_pton (AF_INET, destination, &sa.sin_addr.s_addr);
  //
  // Put some data inside the packet header.
  //
  icmp_packet_send.echo.identifier = (uint16_t) rand ();
  icmp_packet_send.echo.sequence   = 0;
  icmp_packet_send.icmp.type            = 8;       // Echo request.
  icmp_packet_send.icmp.code            = 0;
  icmp_packet_send.icmp.checksum        = 0;
  //
  // Calculate the ICMP checksum.
  //
  checksum = ~(ip_chk ((void *) &icmp_packet_send,
                       sizeof (xicmp_packet_t)));
  icmp_packet_send.icmp.checksum = htons (checksum);
  //
  // Open the socket.
  //
  sfdn = socket (AF_INET, SOCK_RAW, IPPROTO_ICMP);
  if (sfdn < 0)
    {
      perror (NULL);
      return (0);
    }
  //
  // Connect the `sa' destination
  //
  status = connect (sfdn, (struct sockaddr *) &sa, sizeof (sa));
  if (status < 0)
    {
      perror (NULL);
      close (sfdn);
      return (0);
    }
  //
  // Send one single packet: please notice that we send an ICMP,
  // but we receive a full IP.
  //
  for (; retry > 0; retry--)
    {
      sent = send (sfdn, &icmp_packet_send, sizeof (xicmp_packet_t), 0);
      if (sent > 0) break;
      sleep (1);
    }
  if (retry == 0)
    {
      perror (NULL);
      return (1);
    }
  //
  // Packet sent.
  //
  printf ("ping: ");
  //
  // Now receive all ICMP raw packets, and select the one with the same
  // identifier.
  //
  while (1)
    {
      received = read (sfdn, &ip_packet_receive, sizeof (xip_packet_t));

      if (ip_packet_receive.echo.identifier
          == icmp_packet_send.echo.identifier)
       {
         printf ("pong\n");
         break;
       }
    }

  close (sfdn);
  return (0);
}
//----------------------------------------------------------------------
static uint16_t
ip_chk (uint16_t *data, size_t size)
{
  int      i;
  uint32_t sum;
  uint16_t carry;
  uint16_t checksum;
  uint16_t last;
  uint8_t *octet = (uint8_t *) data;
  //
  // 2's complement sum.
  //
  for (i = 0, sum = 0; i < (size/2); i++)
    {
      sum += ntohs (data[i]);
    }
  //
  if (size % 2)
    {
      //
      // The size is odd, and the last octet must be accounted too.
      //
      last = octet[size-1];
      last = last << 8;
      //
      sum += last;
    }
  //
  // Extract the carries and make the checksum.
  //
  carry    = sum >> 16;
  checksum = sum & 0x0000FFFF;
  checksum += carry;
  //
  // End of job.
  //
  return (checksum);
}
//----------------------------------------------------------------------
static void
usage (void)
{
  fprintf (stderr, "Usage: ping IPv4\n");
}
