#include <kernel/driver/ata.h>
#include <kernel/lib_s.h>
#include <kernel/ibm_i386.h>
#include <stdint.h>
#include <errno.h>
//----------------------------------------------------------------------
int
ata_drq (int drive, clock_t timeout)
{
    clock_t       time_start;
    clock_t       time_now;
    clock_t       time_elapsed;
    int           status;
    //
    // The timeout value must be at least two.
    //
    if (timeout < 2)
      {
        timeout = 2;
      }
    //
    // Get the status register.
    //
    time_elapsed       = 0;
    time_start         = s_clock ((pid_t) 0);
    while (time_elapsed < timeout)
      {
        time_now = s_clock ((pid_t) 0);
        time_elapsed = time_now - time_start;
        //
        status = in_8 (ata_table[drive].r_status);
        //
        // Is it BSY?
        //
        if (status & ATA_STATUS_BSY)
          {
            //
            // Read the status again.
            //
            continue;
          }
        //
        // No more busy, but check for errors.
        //
        if (status & ATA_STATUS_DF)
          {
            k_printf ("[%s] ERROR: drive %i fault\n", __func__, drive);
            ata_reset (drive);
            errset (E_HARDWARE_FAULT);
            return (-1);
          }
        //
        if (status & ATA_STATUS_ERR)
          {
            k_printf ("[%s] ERROR: drive %i error\n", __func__, drive);
            ata_reset (drive);
            errset (E_DRIVER_FAULT);
            return (-1);
          }
        //
        // Now check for the DRQ.
        //
        if (status & ATA_STATUS_DRQ)
          {
            //
            // Ok.
            //
            return (0);
          }
      }
    //
    // Sorry: time out!
    //
    k_printf ("[%s] ERROR: drive %i timeout\n", __func__, drive);
    errset (ETIME);
    return (-1);
}
