#ifndef _KERNEL_DRIVER_SCREEN_H
#define _KERNEL_DRIVER_SCREEN_H 1
//----------------------------------------------------------------------
#include <restrict.h>
#include <stdint.h>
//----------------------------------------------------------------------
// Virtual consoles data and VGA references.
//
#define SCREEN_MAX     4
#define SCREEN_ROWS    25
#define SCREEN_COLS    80
#define SCREEN_CELLS  (SCREEN_ROWS * SCREEN_COLS)
//
#define VGA_ATTR       0x07
#define VGA_ADDR       0xB8000
#define VGA_CELL      ((uint16_t *) VGA_ADDR)
//
typedef struct {
    uint16_t  cell[SCREEN_CELLS]; // [1]
    int       position;
} screen_t;
//
// [1] Every character on the screen needs another attribute byte.
//
//----------------------------------------------------------------------
extern int      screen_active;
extern screen_t screen_table[];
//----------------------------------------------------------------------
int       screen_clear    (screen_t *screen);
screen_t *screen_current  (void);
void      screen_init     (void);
int       screen_new_line (screen_t *screen);
int       screen_number   (screen_t *screen);
screen_t *screen_pointer  (int scrn);
int       screen_putc     (screen_t *screen, int c);
int       screen_scroll   (screen_t *screen);
int       screen_select   (screen_t *screen);
void      screen_update   (screen_t *screen);
//----------------------------------------------------------------------
#define screen_cell(c, attrib) ((uint16_t) c \
                             | ((((uint16_t) attrib) << 8) & 0xFF00))
//----------------------------------------------------------------------
#endif

