#include <kernel/fs.h>
#include <errno.h>
#include <kernel/dev.h>
//----------------------------------------------------------------------
int
inode_save (inode_t *inode)
{
    size_t            size;
    unsigned long int start;
    ssize_t           n;
    //
    // Check for valid argument.
    //
    if (inode == NULL)
      {
        errset (EINVAL);                // Invalid argument.
        return (-1);
      }
    //
    // If the inode number is zero, no file system is involved!
    //
    if (inode->ino == 0)
      {
        return (0);
      }
    //
    // Save the super block to disk.
    //
    sb_save (inode->sb);
    //
    // Save the inode to disk.
    //
    if (inode->changed)
      {
        size   = offsetof (inode_t, sb);
        //
        // Calculating start position for write.
        //
        // [1] Boot block.
        // [2] Super block.
        // [3] Inode bit map.
        // [4] Zone bit map.
        // [5] Previous inodes: consider that the inode zero is
        //     present in the inode map, but not in the inode
        //     table.
        //
        start  = 1024;                                  // [1]
        start += 1024;                                  // [2]
        start += (inode->sb->map_inode_blocks * 1024);  // [3]
        start += (inode->sb->map_zone_blocks * 1024);   // [4]
        start += ((inode->ino -1) * size);              // [5]
        //
        // Write the inode.
        //
        n = dev_io ((pid_t) -1, inode->sb->device, DEV_WRITE, start,
                    inode, size, NULL);
        //
        inode->changed = 0;
      }
    return (0);
}
