#include <kernel/lib_k.h>
#include <kernel/ibm_i386.h>
//----------------------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------------------
void
idt_irq_remap (unsigned int offset_1, unsigned int offset_2)
{
    //
    // PIC_P è il PIC primario o «master»;
    // PIC_S è il PIC secondario o «slave».
    //
    // Quando si manifesta un IRQ che riguarda il PIC secondario,
    // il PIC primario riceve IRQ 2.
    //
    // ICW = initialization command word.
    // OCW = operation command word.
    //
    if (DEBUG)
      {
        k_printf ("[%s] PIC (programmable interrupt controller) "
                  "remap: ", __func__);
      }
    //
    out_8 (0x20, 0x10 + 0x01);   // Initialization: 0x10 means that is
    out_8 (0xA0, 0x10 + 0x01);   // is ICW1; 0x01 means that must
                                 // continue up to ICW4.
    if (DEBUG)
      {
        k_printf ("ICW1");
      }
    out_8 (0x21, offset_1);      // ICW2: PIC_P starting at «offset_1».
    out_8 (0xA1, offset_2);      //       PIC_S starting at «offset_2».
    if (DEBUG)
      {
        k_printf (", ICW2");
      }
    out_8 (0x21, 0x04);          // ICW3 PIC_P: IRQ2 driven from PIC_S.
    out_8 (0xA1, 0x02);          // ICW3 PIC_S: driving IRQ2 from PIC_P.
    if (DEBUG)
      {
        k_printf (", ICW3");
      }
    out_8 (0x21, 0x01);          // ICW4: si precisa solo la modalità
    out_8 (0xA1, 0x01);          // del microprocessore; 0x01 = 8086.
    if (DEBUG)
      {
        k_printf (", ICW4");
      }
    out_8 (0x21, 0x00);          // OCW1: reset mask to enable all
    out_8 (0xA1, 0x00);          // IRQ numbers.
    if (DEBUG)
      {
        k_printf (", OCW1.\n");
      }
}
