#include <kernel/proc.h>
#include <kernel/lib_s.h>
#include <kernel/lib_k.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <arpa/inet.h>
//----------------------------------------------------------------------
int
s_connect (pid_t pid, int sfdn, const struct sockaddr *addr,
           socklen_t size)
{
  fd_t               *sfd;
  struct sockaddr_in *sin;
  //
  // Get file descriptor and verify that it is a socket.
  //
  sfd = fd_reference (pid, &sfdn);
  if (sfd == NULL || sfd->file == NULL)
    {
      errset (EBADF);		// Bad file descriptor.
      return (-1);
    }
  if (sfd->file->sock == NULL)
    {
      errset (ENOTSOCK);		// Not a socket.
      return (-1);
    }
  //
  // Verify to have a valid address pointer.
  //
  if (addr == NULL)
    {
      errset (EINVAL);
      return (-1);
    }
  //
  // Check minimal address size.
  //
  if (size < sizeof (struct sockaddr))
    {
      errset (EINVAL);
      return (-1);
    }
  //
  //
  //
  if (addr->sa_family == AF_INET)
    {
      sin = (struct sockaddr_in *) addr;
      //
      // Check to have the destination IP address.
      //
      if (sin->sin_addr.s_addr == 0)
        {
          //
          // This is not valid.
          //
          errset (EADDRNOTAVAIL);
          return (-1);
        }
      //
      // Here should check if the connection can be made, but at
      // the moment only raw connections can be established. So
      // no other checks are made here.
      //
      sfd->file->sock->family = sin->sin_family;
      sfd->file->sock->addr   = ntohl (sin->sin_addr.s_addr);
      sfd->file->sock->port   = sin->sin_port;
    }
  else
    {
      //
      // Unsupported family.
      //
      errset (EAFNOSUPPORT);
      return (-1);
    }
  //
  // Ok.
  //
  return (0);
}
