#include <kernel/proc.h>
#include <kernel/lib_s.h>
#include <kernel/fs.h>
#include <errno.h>
#include <fcntl.h>
//----------------------------------------------------------------------
int
s_fcntl (pid_t pid, int fdn, int cmd, int arg)
{
  proc_t  *ps;
  int      mask;
  //
  // Get process.
  //
  ps = proc_reference (pid);
  //
  // Verify if the file descriptor is valid.
  //
  if (ps->fd[fdn].file == NULL)
    {
      errset (EBADF);		// Bad file descriptor.
      return (-1);
    }
  //
  //
  //
  switch (cmd)
    {
    case F_DUPFD:
      return (fd_dup (pid, fdn, arg));
      break;
    case F_GETFD:
      return (ps->fd[fdn].fd_flags);
      break;
    case F_SETFD:
      ps->fd[fdn].fd_flags = arg;
      return (0);
    case F_GETFL:
      return (ps->fd[fdn].fl_flags);
    case F_SETFL:
      //
      // Calculate a mask with bits that are not to be set.
      //
      mask = (O_ACCMODE | O_CREAT | O_EXCL | O_NOCTTY | O_TRUNC);
      //
      // Set to zero the bits that are not to be set from
      // the argument.
      //
      arg = (arg & ~mask);
      //
      // Set to zero the bit that *are* to be set.
      //
      ps->fd[fdn].fl_flags &= mask;
      //
      // Set the bits, already filtered inside the argument.
      //
      ps->fd[fdn].fl_flags |= arg;
      //
      return (0);
    default:
      errset (EINVAL);		// Not implemented.
      return (-1);
    }
}
