#include <kernel/net.h>
#include <kernel/net/ethernet.h>
#include <kernel/net/route.h>
#include <kernel/net/arp.h>
#include <errno.h>
#include <kernel/proc.h>
//----------------------------------------------------------------------
// This syscall is present only inside os32.
//----------------------------------------------------------------------
int
s_ipconfig (pid_t pid, int n, in_addr_t addr, int m)
{
  //
  // Must be a privileged process.
  //
  if (proc_table[pid].euid != 0)
    {
      errset (EPERM);
      return (-1);
    }
  //
  // Check arguments: net0 cannot be modified, because it is necessarily
  // assigned to the loopback virtual interface.
  //
  if (n > NET_MAX_DEVICES || n < 1)
    {
      errset (EINVAL);
      return (-1);
    }
  if (m > 32 || n < 0)
    {
      errset (EINVAL);
      return (-1);
    }
  //
  // Verify that the interface is present.
  //
  if (net_table[n].type == NET_DEV_NULL)
    {
      errset (ENODEV);
      return (-1);
    }
  //
  // Remove previous route related to the interface, if the interface
  // was already configured.
  //
  if (net_table[n].ip != 0 && net_table[n].m != 0)
    {
      s_routedel (pid, net_table[n].ip, net_table[n].m);
    }
  //
  // Modify the net_table[].
  //
  net_table[n].ip = ntohl (addr);
  net_table[n].m  = m;
  //
  // Add the route.
  //
  if (net_table[n].ip != 0 && net_table[n].m != 0)
    {
      return (s_routeadd (pid, addr, m, 0, n));
    }
  //
  return (0);
}
