#include <kernel/net.h>
#include <kernel/net/route.h>
#include <errno.h>
#include <arpa/inet.h>
#include <sys/os32.h>
//----------------------------------------------------------------------
ssize_t
s_send (pid_t pid, int sfdn, const void *buffer, size_t size, int flags)
{
  fd_t     *sfd;
  h_addr_t  src;
  int       status;
  //
  // Get file descriptor and verify that it is a socket.
  //
  sfd = fd_reference (pid, &sfdn);
  if (sfd == NULL || sfd->file == NULL)
    {
      errset (EBADF);		// Bad file descriptor.
      return ((ssize_t) -1);
    }
  if (sfd->file->sock == NULL)
    {
      errset (ENOTSOCK);	// Not a socket.
      return ((ssize_t) -1);
    }
  //
  // Verify to have a valid buffer pointer.
  //
  if (buffer == NULL)
    {
      errset (EINVAL);
      return ((ssize_t) -1);
    }
  //
  //
  //
  if (sfd->file->sock->family == AF_INET)                    // INET
    {
      //
      // AF_INET requires at least the destination address.
      //
      if (sfd->file->sock->addr == 0)
        {
          errset (EDESTADDRREQ);
          return ((ssize_t) -1);
        }
      //
      // Get the source address, from the routing table, based
      // on the destination.
      //
      src = route_remote_to_local (sfd->file->sock->addr);
      if (src == ((h_addr_t) -1))
        {
          errset (errno);
          return ((ssize_t) -1);
        }
      //
      if (sfd->file->sock->type == SOCK_RAW)                 // RAW
        {
          if (sfd->file->sock->protocol == IPPROTO_ICMP)     // ICMP
            {
              status = ip_tx (src, sfd->file->sock->addr,
                              sfd->file->sock->protocol,
                              buffer, size);
              if (status)
                {
                  errset (errno);
                  return ((ssize_t) -1);
                }
              else
                {
                  return ((ssize_t) size);
                }
            }
          else
            {
              //
              // Unsupported protocol.
              //
              errset (EPROTONOSUPPORT);
              return ((ssize_t) -1);
            }
        }
      else
        {
          //
          // Unsupported type.
          //
          errset (EPROTONOSUPPORT);
          return ((ssize_t) -1);
        }
    }
  else
    {
      //
      // Unsupported family.
      //
      errset (EAFNOSUPPORT);
      return ((ssize_t) -1);
    }
}
