#include <kernel/proc.h>
#include <kernel/lib_s.h>
#include <kernel/lib_k.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <arpa/inet.h>
//----------------------------------------------------------------------
int
s_socket (pid_t pid, int family, int type, int protocol)
{
  fd_t    *fd;
  int      sfdn;
  file_t  *file;
  sock_t  *sock;
  //
  // Check supported family type.
  //
  if (family != AF_INET)
    {
      errset (EAFNOSUPPORT);
      return (-1);
    }
  //
  // Check supported communication type.
  //
  if (type != SOCK_RAW)
    {
      errset (EPROTONOSUPPORT);
      return (-1);
    }
  //
  // Check supported protocol type.
  //
  if (protocol != IPPROTO_ICMP)
    {
      errset (EPROTONOSUPPORT);
      return (-1);
    }
  //
  // If it is a raw socket, must be a privileged process.
  //
  if (type == SOCK_RAW && proc_table[pid].euid != 0)
    {
      errset (EACCES);
      return (-1);
    }
  //
  // Find a free slot inside the sock_table[].
  //
  sock = sock_reference (-1);
  if (sock == NULL)
    {
      errset (ENFILE);
      return (-1);
    }
  //
  // Find a free slot inside the file table.
  //
  file = file_reference (-1);
  if (file == NULL)
    {
      errset (ENFILE);          // Too many files open in system.
      return (-1);
    }
  //
  // Find a free slot inside the file descriptor table.
  // Variable `sfdn' will be modified by the call to `fd_reference()'.
  //
  sfdn = -1;
  fd = fd_reference (pid, &sfdn);
  if (fd == NULL)
    {
      errset (EMFILE);          // Too many open files.
      return (-1);
    }
  //
  // socket, system file and file descriptor ready: put data inside
  // them.
  //
  sock->active   = 1;
  sock->family   = family;
  sock->type     = type;
  sock->protocol = protocol;
  sock->addr     = 0;
  memset (sock->read.clock, 0x00, sizeof (sock->read.clock));
  //
  file->references = 1;
  file->oflags     = O_RDWR;
  file->inode      = NULL;
  file->sock       = sock;
  file->offset     = 0;
  //
  fd->fl_flags     = (O_RDWR | O_APPEND);
  fd->fd_flags     = 0;
  fd->file         = file;
  //
  // Return the file descriptor.
  //
  return (sfdn);
}
