#include <kernel/main.h>
#include <kernel/main/build.h>
#include <kernel/lib_k.h>
#include <kernel/driver/tty.h>
#include <kernel/memory.h>
#include <stdlib.h>
#include <stdint.h>
#include <kernel/driver/screen.h>
#include <kernel/proc.h>
#include <kernel/lib_s.h>
#include <kernel/fs.h>
#include <unistd.h>
#include <stdint.h>
#include <kernel/driver/kbd.h>
#include <kernel/driver/ata.h>
#include <kernel/dev.h>
#include <kernel/blk.h>
#include <fcntl.h>
#include <string.h>
#include <limits.h>
#include <kernel/driver/pci.h>
#include <kernel/net/ethernet.h>
#include <kernel/net/icmp.h>
#include <kernel/net/arp.h>
#include <kernel/net/route.h>
#include <kernel/driver/nic/ne2k.h>
//----------------------------------------------------------------------
static char command[MAX_CANON];
//----------------------------------------------------------------------
void
kmain (uint32_t magic, multiboot_t *mboot_data)
{
    int          exit;
    pid_t        pid;
    int          counter;
    char        *exec_argv[2];
    int          status;
    ssize_t      count;
    //
    h_addr_t     ip_to_be_found = 0xAC15FEFE; //172.21.254.254
    //
    // Reset video and select the initial console.
    //
    tty_init ();
    //
    // Verify `multiboot' data.
    //
    if (magic == 0x2BADB002)
      {
        //
        // Save multiboot data.
        //
        mboot_save (mboot_data);
        //
        // Show compilation date and time, plus upper memory limit.
        //
        k_printf ("os32 build %s ram %i Kibyte\n",
                  BUILD_DATE, (int) multiboot.mem_upper);
        //
        // Show also the command line.
        //
        k_printf ("%s\n", multiboot.cmdline);
        //
        // Set `mb_max', for memory allocation.
        //
        mb_size (multiboot.mem_upper * 1024);
        //
        // keyboard initialization.
        //
        kbd_load ();
        //
        // Block cache initialization.
        //
        blk_cache_init ();
        //
        // PCI bus initialization.
        //
        pci_init ();
        //
        // File system management initialization.
        //
        fs_init ();
        //
        // Set up processes.
        //
        proc_init ();
        //
        // Set up network.
        //
        net_init ();
      }
    else
      {
        //
        // If it is not a multiboot loader, it is an error.
        //
        k_printf ("os32 build %s. ERROR. no \"multiboot\" header!\n",
                  BUILD_DATE);
        k_printf ("[%s] system halted\n", __func__);
        k_exit ();
      }
    //
    // The kernel will run interactively.
    //
    k_printf (
".------------------------------------------------------------------.\n"
"| `h' followed by [Enter] to show a menu                           |\n"
"`------------------------------------------------------------------'\n"
);
    // menu ();
    //
    //
    //
    for (exit = 0; exit == 0;)
      {
        //
        // While in kernel code, timer interrupt don't start the
        // scheduler. The kernel must leave control to the scheduler
        // via a null system call.
        //
        sys (SYS_0, NULL, 0);
        //
        // Back to work: read the keyboard from the TTY device.
        //
        count = dev_io ((pid_t) 0, (dev_t) DEV_TTY, DEV_READ, (off_t) 0,
                        command, (size_t) MAX_CANON, NULL);
        //
        // Check if there is a command: the kernel does not
        // go to sleep.
        //
        if (count == 0)
          {
            //
            // No command is ready in the buffer keyboard.
            //
            continue;
          }
        if (count == 1)
          {
            //
            // Just [Enter] or [Ctrl d] was pressed.
            //
            continue;
          }
        if (count > MAX_CANON)
          {
            //
            // Something impossible!
            //
            continue;
          }
        //
        // The last character is the "line delimiter" (new line et al.),
        // or zero in case of EOF. The last character is replaced with
        // zero, so that the command becomes a terminated string.
        //
        command[count-1] = 0;
        //
        // A command was typed: start to check what it was.
        //
        if      (strncmp (command, "1", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 1, SIGKILL);     // kill init.
          }
        else if (strncmp (command, "2", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 2,  SIGTERM);    // kill proc. 2
          }
        else if (strncmp (command, "3", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 3,  SIGTERM);    // kill proc. 3
          }
        else if (strncmp (command, "4", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 4,  SIGTERM);    // kill proc. 4
          }
        else if (strncmp (command, "5", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 5,  SIGTERM);    // kill proc. 5
          }
        else if (strncmp (command, "6", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 6,  SIGTERM);    // kill proc. 6
          }
        else if (strncmp (command, "7", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 7,  SIGTERM);    // kill proc. 7
          }
        else if (strncmp (command, "8", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 8,  SIGTERM);    // kill proc. 8
          }
        else if (strncmp (command, "9", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 9,  SIGTERM);    // kill proc. 9
          }
        else if (strncmp (command, "A", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 10,  SIGTERM);   // kill proc. 10
          }
        else if (strncmp (command, "B", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 11,  SIGTERM);   // kill proc. 11
          }
        else if (strncmp (command, "C", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 12,  SIGTERM);   // kill proc. 12
          }
        else if (strncmp (command, "D", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 13,  SIGTERM);   // kill proc. 13
          }
        else if (strncmp (command, "E", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 14,  SIGTERM);   // kill proc. 14
          }
        else if (strncmp (command, "F", MAX_CANON) == 0)
          {
            s_kill ((pid_t) 0, (pid_t) 15,  SIGTERM);   // kill proc. 15
          }
        else if (strncmp (command, "a", MAX_CANON) == 0)
          {
            run ("/bin/aaa", NULL, NULL);
          }
        else if (strncmp (command, "b", MAX_CANON) == 0)
          {
            run ("/bin/bbb", NULL, NULL);
          }
        else if (strncmp (command, "c", MAX_CANON) == 0)
          {
            run ("/bin/ccc", NULL, NULL);
          }
        else if (strncmp (command, "f", MAX_CANON) == 0)
          {
            pid = fork ();
            if (pid == -1)
              {
                k_perror (NULL);
              }
            else if (pid == 0)
              {
                //
                // Get child real pid.
                //
                pid = getpid ();
                //
                // Please note that the child is no more a kernel, and
                // can access to process system calls.
                //
                for (counter = 0; counter < 60; counter++)
                  {
                    z_printf ("%1x", (int) pid);
                    sleep (1);
                  }
              }
            else
              {
                z_printf ("io sono il genitore di %i\n", pid);
              }
          }
        else if (strncmp (command, "g", MAX_CANON) == 0)
          {
            gdt_print (&gdt_register, 0, 20);
          }
        else if (strncmp (command, "G", MAX_CANON) == 0)
          {
            gdt_print (&gdt_register, 21, 41);
          }
        else if (strncmp (command, "h", MAX_CANON) == 0)
          {
            menu ();
          }
        else if (strncmp (command, "i", MAX_CANON) == 0)
          {
            idt_print (&idt_register, 0, 20);
          }
        else if (strncmp (command, "I", MAX_CANON) == 0)
          {
            idt_print (&idt_register, 21, 41);
          }
        else if (strncmp (command, "m", MAX_CANON) == 0)
          {
            mb_print ();
          }
        else if (strncmp (command, "n", MAX_CANON) == 0)
          {
            inode_print ();
          }
        else if (strncmp (command, "p", MAX_CANON) == 0)
          {
            proc_print ();
          }
        else if (strncmp (command, "s", MAX_CANON) == 0)
          {
            sb_print ();
          }
        else if (strncmp (command, "t", MAX_CANON) == 0)
          {
            k_printf ("clock: %lli time: %i\n",
                      (long long int) k_clock (),
                      (int) k_time (NULL));
          }
        else if (strncmp (command, "T", MAX_CANON) == 0)
          {
            while (1)
              {
                k_printf ("clock: %lli time: %i\n",
                      (long long int) k_clock (),
                      (int) k_time (NULL));
              }
          }
        else if (strncmp (command, "w", MAX_CANON) == 0)
          {
            status = s_open ((pid_t) 0, "/tmp/test",
                             O_WRONLY|O_CREAT|O_TRUNC, 0644);
            //
            if (status >= 0)
              {
                status = s_close ((pid_t) 0, status);
                if (status != 0)
                  {
                    k_perror (NULL);
                  }
              }
          }
        else if (strncmp (command, "x", MAX_CANON) == 0)
          {
            //
            // Load init.
            //
            exec_argv[0] = "/bin/init";
            exec_argv[1] = NULL;
            pid = run ("/bin/init", exec_argv, NULL);
            //
            // Just sleep.
            //
            while (1)
              {
                sys (SYS_0, NULL, 0);
              }
          }
        else if (strncmp (command, "q", MAX_CANON) == 0)
          {
            k_printf ("System halted!\n");
            return;
          }
        else if (strncmp (command, "y", MAX_CANON) == 0)
          {
            //icmp_test3 ();
            //icmp_test2 ();
            ip_test2 ();
            //ip_test ();
          }
        else if (strncmp (command, "r", MAX_CANON) == 0)
          {
            arp_print ();
          }
        else if (strncmp (command, "R", MAX_CANON) == 0)
          {
            arp_request (ip_to_be_found);
          }
      }
}
