#include <stddef.h>
#include <kernel/multiboot.h>
#include <kernel/lib_k.h>
#include <string.h>
#include <errno.h>
//----------------------------------------------------------------------
char **
mboot_cmdline_opt (const char *opt, const char *delim)
{
  static char  option[MBOOT_CMDLINE_OPTION_MAX];
  static char *argument[MBOOT_CMDLINE_ARGUMENTS_MAX];
         char *a;
         char *z;
         char *t;
          int  i;
       size_t  size;
  //
  // Check input.
  //
  if (opt == NULL)
    {
      errset (EINVAL);
      return (NULL);
    }
  //
  // Find the option.
  //
  a = strstr (multiboot.cmdline, opt);
  if (a == NULL)
    {
      return (NULL);
    }
  //
  // Find the end of the option: might be a space or the end of the
  // string.
  //
  z = strpbrk (a, " \t\n");
  //
  // Copy the option inside the static array `option[]'.
  //
  if (z == NULL)
    {
      strncpy (option, a, MBOOT_CMDLINE_OPTION_MAX-1);
      option[MBOOT_CMDLINE_OPTION_MAX-1] = 0;
    }
  else
    {
      size = (uintptr_t) z - (uintptr_t) a;
      strncpy (option, a, size);
      option[size] = 0;

//k_printf ("size: %i, optione: \"%s\"\n", (int) size, option);
    }
  //
  // Find the option name, to be saved as the first argument.
  //
  t = strtok (option, "=");
  if (t == NULL)
    {
      errset (EUNKNOWN);
      return (NULL);
    }
  argument[0] = t;
  //
  // If there is no delimiter, replace it with a string containing
  // just a space.
  //
  if (delim == NULL)
    {
      delim = " ";
    }
  //
  for (i = 1; i < MBOOT_CMDLINE_ARGUMENTS_MAX; i++)
    {
      t = strtok (NULL, delim);
      if (t == NULL)
        {
          //
          // The argument will be an empty string, taken from
          // the end of the option string.
          //
          argument[i] = &option[MBOOT_CMDLINE_OPTION_MAX-1];
        }
      else
        {
          argument[i] = t;
        }
    }
  //
  // Return.
  //
  return (argument);
}
