#include <kernel/net.h>
#include <kernel/net/ethernet.h>
#include <kernel/net/arp.h>
#include <kernel/net/ip.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>
#include <errno.h>
#include <arpa/inet.h>
//----------------------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------------------
void
arp_request (h_addr_t ip)
{
  const uint8_t ethernet_broadcast[NET_ETHERNET_ADDRESS_LENGTH]
                     = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
  const uint8_t ethernet_null[NET_ETHERNET_ADDRESS_LENGTH]
                     = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
  //
  net_ethernet_frame_t  frame;
  arp_packet_t         *arp = (arp_packet_t *) &frame.packet;
  int                   n;          // NET table index.
  int                   i;
  //
  // Send the ARP request to all Ethernet interfaces.
  //
  for (n = 0; n < NET_MAX_DEVICES; n++)
    {
      if (net_table[n].type & NET_DEV_ETH)
        {
          //
          // Build the ARP request pachet, starting from Ethernet
          // MAC addresses and protocol type.
          //
          memcpy (frame.header.src, net_table[n].ethernet.mac,
                  (size_t) NET_ETHERNET_ADDRESS_LENGTH);
          memcpy (frame.header.dst, ethernet_broadcast,
                  (size_t) NET_ETHERNET_ADDRESS_LENGTH);
          frame.header.type = htons (NET_PROT_ARP);
          //
          // Now the ARP packet inside.
          //
          arp->hardware_type = htons (ARP_HW_ETHERNET);
          arp->protocol_type = htons (NET_PROT_IP);
          arp->hardware_address_length
                             = NET_ETHERNET_ADDRESS_LENGTH;
          arp->protocol_address_length
                             = NET_IP_ADDRESS_LENGTH;
          arp->opcode        = htons (ARP_TYPE_REQUEST);
          memcpy (arp->sender_mac, net_table[n].ethernet.mac,
                  (size_t) NET_ETHERNET_ADDRESS_LENGTH);
          arp->sender_ip     = htonl (net_table[n].ip);
          memcpy (arp->target_mac, ethernet_null,
                  (size_t) NET_ETHERNET_ADDRESS_LENGTH);
          //
          arp->target_ip     = htonl (ip);
          //
          for (i = 0; i < (sizeof_array (arp->filler)); i++)
            {
              arp->filler[i] = 0;
            }
          //
          // Send it.
          //
          ethernet_tx (n, &frame, sizeof (arp_packet_t) + 14);
        }
    }
}
