#ifndef _KERNEL_NET_ICMP_H
#define _KERNEL_NET_ICMP_H    1
//----------------------------------------------------------------------
#include <stdint.h>
#include <sys/types.h>
#include <kernel/net/ip.h>
//----------------------------------------------------------------------
#define ICMP_HEADER_SIZE  32
#define ICMP_MAX_PACKET_SIZE  NET_IP_MAX_DATA_SIZE
#define ICMP_MAX_DATA_SIZE    ICMP_MAX_PACKET_SIZE-ICMP_HEADER_SIZE
//
#define ICMP_ECHO_HEADER_SIZE    4
#define ICMP_ECHO_MAX_DATA_SIZE  ICMP_MAX_DATA_SIZE \
                                 - ICMP_ECHO_HEADER_SIZE
//----------------------------------------------------------------------
#define ICMP_TYPE_ECHO_REPLY               0
#define ICMP_TYPE_DESTINATION_UNREACHABLE  3
#define ICMP_TYPE_ECHO_REQUEST             8
//----------------------------------------------------------------------
//
// ICMP echo header.
//
typedef struct {
  uint16_t identifier;
  uint16_t sequence;
} __attribute__ ((packed)) icmp_echo_header_t;
//
// ICMP echo packet.
//
typedef struct {
  icmp_echo_header_t header;
  uint8_t  data[ICMP_ECHO_MAX_DATA_SIZE];
} __attribute__ ((packed)) icmp_echo_packet_t;
//
// ICMP header.
//
typedef struct {
  uint8_t  type;
  uint8_t  code;
  uint16_t checksum;
} __attribute__ ((packed)) icmp_header_t;
//
// ICMP packet, for transmission.
//
typedef struct {
  icmp_header_t header;
  union {
    icmp_echo_packet_t echo;
    uint8_t            data[ICMP_MAX_DATA_SIZE];
  };
} __attribute__ ((packed)) icmp_packet_t;

//----------------------------------------------------------------------
int      icmp_echo_tx (h_addr_t src, h_addr_t dst,
                       int type, int code,
                       int identifier, int sequence,
                       uint8_t *data, size_t size);
//
int      icmp_tx      (h_addr_t src, h_addr_t dst,
                       int type, int code, icmp_packet_t *icmp,
                       size_t size);
//
int      icmp_rx      (int i);


void     icmp_test2   (void);
void     icmp_test3   (void);


//----------------------------------------------------------------------
#endif
