#include <sys/os32.h>
#include <kernel/lib_k.h>
#include <arpa/inet.h>
#include <kernel/net.h>
#include <kernel/net/ip.h>
#include <kernel/net/icmp.h>
#include <errno.h>
//----------------------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------------------
int
icmp_rx (int i)
{
  icmp_packet_t      *icmp;
  size_t              size;
  //
  //
  //
  if ((i >= IP_MAX_PACKETS) || i < 0)
    {
      errset (EINVAL);
      return (-1);
    }
  //
  //
  //
  icmp = (icmp_packet_t *) ip_table[i].pdu4;
  size = ntohs (ip_table[i].packet.header.length)
          - sizeof (icmp_header_t);
  //
  // This function is used by the kernel, to do something automatically,
  // when some ICMP packets arrive. The kernel does not remove the
  // serviced packets, but must remember what it already have seen.
  // If it finds that the packet clock time stamp is the same as
  // the value saved for the kernel, there is nothing more to be done.
  //
  if (ip_table[i].kernel_serviced == ip_table[i].clock)
    {
      return (0);
    }
  //
  //
  //
  if (icmp->header.type == ICMP_TYPE_ECHO_REQUEST)
    {
      size -= sizeof (icmp_header_t);
      //
      // Reply: please note that source and destination IP addresses
      // are now inverted.
      //
      icmp_echo_tx (ntohl (ip_table[i].packet.header.dst),
                    ntohl (ip_table[i].packet.header.src),
                    ICMP_TYPE_ECHO_REPLY, 0,
                    ntohs (icmp->echo.header.identifier),
                    ntohs (icmp->echo.header.sequence),
                    icmp->echo.data, size);
      //
      // ICMP echo request are resolved internally, but the packet
      // might be read from the RAW socket too. So it is not removed
      // from the ip_table[].
      //
      ip_table[i].kernel_serviced = ip_table[i].clock;
    }
  else
    {
      //
      // No other ICMP type is handled at the moment, but keep
      // the packet, because might be read from a RAW ICMP socket.
      //
      ;
    }
  //
  return (0);
}
