#ifndef _KERNEL_NET_IP_H
#define _KERNEL_NET_IP_H    1
//----------------------------------------------------------------------
#include <stdint.h>
#include <sys/types.h>
#include <kernel/net.h>
//----------------------------------------------------------------------
#define IP_VERSION          4
#define IP_TTL              64
//----------------------------------------------------------------------
#define IP_MAX_PACKETS      64   // era 64
//----------------------------------------------------------------------
#define IP_PROTOCOL_ICMP    1
#define IP_PROTOCOL_TCP     6
#define IP_PROTOCOL_UDP     147
//----------------------------------------------------------------------
//
// IP packet, for transmission (no options here).
// It is just the same as `ip_header_t', with the
// `data[]' member addition.
//
typedef struct {
  uint8_t   header_length : 4,
            version       : 4;
  uint8_t   service;
  uint16_t  length;
  uint16_t  id;
  uint16_t  fragment;    // [flg][fragment_offset] 3+13 bit
  uint8_t   ttl;
  uint8_t   protocol;
  uint16_t  header_checksum;
  in_addr_t src;   // Network byte order.
  in_addr_t dst;   // Network byte order.
  uint8_t   data[NET_IP_MAX_DATA_SIZE];
} __attribute__ ((packed)) ip_packet_t;
//
// IP table for IP packets.
//
typedef struct {
  clock_t   clock;           // [1]
  clock_t   kernel_serviced; // [2]
  uint8_t  *pdu4;
  union {
    uint8_t         octet[NET_IP_MAX_PACKET_SIZE];
    net_ip_header_t header;
  } packet;
} ip_t;
//
// [1] This is the arrival packet clock time stamp. When a new packet
//     is to be saved inside the ip_table[], the older packet is
//     replaced, even if it was not used. No packets are removed,
//     because they might be read from a RAW socket, for some reason.
//
// [2] This is only for kernel usage, when a connection is established
//     by the kernel, without a socket. The member kernel_serviced
//     is used to remember to have see a certain packet and that the
//     kernel does not have to try to service it again. For example,
//     when an ECHO REQUEST packet arrive, the kernel answares with
//     an ECHO REPLY, but does not remove the packet that might be
//     read from a true RAW socket. So, the kernel writes inside
//     kernel_serviced the same value found inside clock,
//     so that it know that it was already read.
//
//
// External IP table data.
//
extern ip_t ip_table[IP_MAX_PACKETS];
//----------------------------------------------------------------------
uint16_t ip_checksum  (uint16_t *data, size_t size);
int      ip_rx        (int eth, int f);
ip_t    *ip_reference (void);
void     ip_test      (void);
void     ip_test2     (void);
ssize_t  ip_header    (h_addr_t src, h_addr_t dst, uint16_t id,
                       uint8_t ttl, uint8_t protocol, void *buffer,
                       size_t length);
int      ip_tx        (h_addr_t src, h_addr_t dst, int protocol,
                       const void *buffer, size_t size);
//----------------------------------------------------------------------
#endif
