#include <kernel/net.h>
#include <kernel/net/ip.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>
#include <errno.h>
#include <arpa/inet.h>
//----------------------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------------------
ssize_t
ip_header (h_addr_t src, h_addr_t dst, uint16_t id, uint8_t ttl,
           uint8_t protocol, void *buffer, size_t length)
{
  net_ip_header_t header;
  uint16_t        checksum;
  //
  // Check size.
  //
  if (length < sizeof (net_ip_header_t))
    {
      errset (EINVAL);
      return ((ssize_t) -1);
    }
  //
  // Prepare the header.
  //
  header.version         = IP_VERSION;
  header.ihl             = sizeof (net_ip_header_t) / 4;
  header.service         = 0x00;               // Routine, normal.
  header.length          = htons (length);
  header.id              = htons (id);
  header.fragment        = htons (0x4000);      // Do not fragment!
  header.ttl             = ttl;
  header.protocol        = protocol;
  header.checksum        = 0;
  header.src             = htonl (src);
  header.dst             = htonl (dst);
  //
  // Fix the length.
  //
  length = sizeof (net_ip_header_t);
  //
  // Now set the header checksum.
  //
  checksum        = ~(ip_checksum ((void *) &header, length));
  header.checksum = htons (checksum);
  //
  // Copy the header to the buffer
  //
  memcpy (buffer, &header, length);
  //
  // Return size written.
  //
  return (length);
}
