#include <kernel/net.h>
#include <kernel/net/ethernet.h>
#include <kernel/net/route.h>
#include <kernel/net/arp.h>
#include <kernel/lib_s.h>
#include <kernel/proc.h>
#include <kernel/multiboot.h>
#include <stdlib.h>
#include <kernel/lib_k.h>
#include <string.h>
#include <errno.h>
//----------------------------------------------------------------------
void
net_init (void)
{
  int n;                // NET device table index.
  int b;                // Buffer NET device index.
  int i;
  char *net   = "net0";
  char *route = "route0";
  char **argument;
  in_addr_t ip_a;
  in_addr_t ip_b;
  int status;
  //
  // Reset the NET device table.
  //
  for (n = 0; n < NET_MAX_DEVICES; n++)
    {
      net_table[n].type = NET_DEV_NULL;
    }
  //
  // Set up the loopback interface.
  //
  net_table[0].type    = NET_DEV_LOOPBACK;
  net_table[0].ip      = INADDR_LOOPBACK;
  net_table[0].m       = 8;
  //
  for (b = 0; b < NET_MAX_BUFFERS; b++)
    {
      net_table[0].loopback.buffer[b].clock = 0;
    }
  //
  // Prepare ARP table
  //
  arp_init ();
  //
  // Add Ethernet devices, but starting from the second interface
  // inside the net_table[].
  //
  ethernet_init (1);
  //
  // Prepare routes.
  //
  route_init ();
  route_sort ();
  //
  // Command line options: counter `i' is scanned like a character.
  //
  for (i = '0'; i <= '9'; i++)
    {
      net[3] = i;
      argument = mboot_cmdline_opt (net, ",");
      if (argument != NULL)
        {
          //
          status = inet_pton (AF_INET, argument[2], &ip_a);
          if (status != 1)
            {
              continue;
            }
          //
          s_ipconfig ((pid_t) 0, atoi (argument[1]), ip_a,
                      atoi (argument[3]));
        }
    }
  //
  for (i = '0'; i <= '9'; i++)
    {
      route[5] = i;
      argument = mboot_cmdline_opt (route, ",");
      if (argument != NULL)
        {
          //
          status = inet_pton (AF_INET, argument[1], &ip_a);
          if (status != 1)
            {
              continue;
            }
          status = inet_pton (AF_INET, argument[3], &ip_b);
          if (status != 1)
            {
              continue;
            }
          //
          s_routeadd ((pid_t) 0,
                      ip_a, atoi (argument[2]),
                      ip_b, atoi (argument[4]));
        }
    }
  //
  //
  //
  net_print ();
  route_print ();
}
