#include <sys/os32.h>
#include <kernel/net.h>
#include <kernel/net/ip.h>
#include <kernel/net/ethernet.h>
#include <kernel/driver/nic/ne2k.h>
#include <kernel/driver/pci.h>
#include <kernel/ibm_i386.h>
#include <errno.h>
//----------------------------------------------------------------------
int
net_ip_tx (h_addr_t src, h_addr_t dst, const void *packet, size_t size)
{
  int                    s;         // source net interface.
  int                    d;         // destination net interface.
  net_loopback_buffer_t *loopback;
  //
  // Check for PDU size.
  //
  if (size > NET_MTU)
    {
      errset (E_PDU_TOO_BIG);
      return (-1);
    }
  //
  // Find the sender interface.
  //
  s = net_index (src);
  if (s < 0)
    {
      errset (errno);           // ENODEV.
      return (-1);
    }
  //
  // Check if the destination is a local interface.
  //
  d = net_index (dst);
  if (d >= 0)
    {
      //
      // It is a local interface, so must change the destination
      // to the loopback device: it must be `net0'.
      //
      d = 0;
    }
  else
    {
      //
      // Should not be necessary, but for coherence with the rest
      // of the code...
      //
      d = s;
    }
  //
  // Check if the destination is a the loopback interface.
  //
  if (net_table[d].type & NET_DEV_LOOP)
    {
      loopback = net_loopback_buffer (d);
      if (loopback == NULL)
        {
          errset (errno);
          return (-1);
        }
      loopback->clock = k_clock ();
      loopback->size  = size;
      memcpy (&loopback->packet, packet, size);
      return (0);
    }
  //
  // The destination wasn't the loopback interface, so check if the
  // source is an Ethernet device.
  //
  if (net_table[s].type & NET_DEV_ETH)
    {
      return (ethernet_ip_tx (src, dst, packet, size));
    }
  //
  // Should never reach the end, but who knows...
  //
  errset (errno);           // ENODEV.
  return (-1);
}
