#include <arpa/inet.h>
#include <stdint.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
//----------------------------------------------------------------------
#define INET_PTON_MAX_STRING_SIZE 31
//----------------------------------------------------------------------
int
inet_pton (int family, const char *src, void *dst)
{
  char     *t;
  int       ipv4[4];
  int       i;
  in_addr_t result;
  char      source[INET_PTON_MAX_STRING_SIZE+1];
  //
  // Check family type: only IPv4 is available here.
  //
  if (family != AF_INET)
    {
      errset (EAFNOSUPPORT);
      return (-1);
    }
  //
  // Check for NULL pointers.
  //
  if (src == NULL || dst == NULL)
    {
      errset (EINVAL);
      return (-1);
    }
  //
  // Check the source string size.
  //
  if (strlen (src) > INET_PTON_MAX_STRING_SIZE)
    {
      //
      // The IPv4 address scan is finished prematurely:
      // return zero to tell that the address string is
      // not correct.
      //
      return (0);
    }
  //
  // Copy the source address, to be able to modify
  // the string.
  //
  strcpy (source, src);
  //
  // Start ``tokenize'' the string: it is here accepted also
  // the space as a delimiter.
  //
  t = strtok (source, ". ");
  //
  for (i = 0; i < 4 && t != NULL; i++)
    {
      ipv4[i] = atoi (t);
      if (ipv4[i] > 255 || ipv4[i] < 0)
        {
          //
          // An octet cannot have a value  greater than 255,
          // and cannot be negative.
          //
          break;
        }
      t = strtok (NULL, ". ");
    }
  //
  if (i < 4)
    {
      //
      // The IPv4 address scan is finished prematurely:
      // return zero to tell that the address string is
      // not correct.
      //
      return (0);
    }
  //
  // Translate into a network byte order IPv4 address:
  // the architecture is little-endian.
  //
  result = 0;
  result += (ipv4[0] <<  0) & 0x000000FF;
  result += (ipv4[1] <<  8) & 0x0000FF00;
  result += (ipv4[2] << 16) & 0x00FF0000;
  result += (ipv4[3] << 24) & 0xFF000000;
  //
  // Update the destination.
  //
  *((in_addr_t *) dst) = result;
  //
  // Return ok.
  //
  return (1);
}
