#include <sys/os32.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
//----------------------------------------------------------------------
ssize_t
recv (int sfdn, void *buffer, size_t count, int flags)
{
    sysmsg_recv_t msg;
    //
    // Reduce size of read if necessary.
    //
    if (count > BUFSIZ)
      {
        count = BUFSIZ;
      }
    //
    // Fill the message.
    //
    msg.sfdn  = sfdn;
    msg.count = count;
    msg.flags = flags;
    msg.ret   = 0;
    //
    // Repeat syscall, until something is received or end of file is
    // reached.
    //
    while (1)
      {
        sys (SYS_RECV, &msg, (sizeof msg));
        if (msg.ret != 0)
          {
            //
            // If we have received (positive return) or if we
            // had an error (negative return), we exit. Otherwise
            // we wait for a packet.
            //
            break;
          }
      }
    //
    // Before return: be careful with the `msg.buffer' copy, because
    // it cannot be longer than `count', otherwise, some unexpected
    // memory will be overwritten!
    //
    if (msg.ret < 0)
      {
        //
        // No valid read, no change inside the buffer.
        //
        errno = msg.errno;
        errln = msg.errln;
        strncpy (errfn, msg.errfn, PATH_MAX);
        return (msg.ret);
      }
    //
    if (msg.ret > count)
      {
        //
        // A strange value was returned. Considering it a read error.
        //
        errset (EIO);                   // I/O error.
        return (-1);
      }
    //
    // A valid read: fill the buffer with `msg.ret' bytes.
    //
    memcpy (buffer, msg.buffer, msg.ret);
    //
    // Return.
    //
    return (msg.ret);
}
