#ifndef _SYS_STAT_H
#define _SYS_STAT_H     1

#include <restrict.h>
#include <sys/types.h>  // dev_t
                        // off_t
                        // blkcnt_t
                        // blksize_t
                        // ino_t
                        // mode_t
                        // nlink_t
                        // uid_t
                        // gid_t
                        // time_t
//----------------------------------------------------------------------
// File type.
//----------------------------------------------------------------------
#define S_IFMT   0170000        // File type mask.
//
#define S_IFBLK  0060000        // Block device file.
#define S_IFCHR  0020000        // Character device file.
#define S_IFIFO  0010000        // Pipe (FIFO) file.
#define S_IFREG  0100000        // Regular file.
#define S_IFDIR  0040000        // Directory.
#define S_IFLNK  0120000        // Symbolic link.
#define S_IFSOCK 0140000        // Unix domain socket.
//----------------------------------------------------------------------
// Owner user access permissions.
//----------------------------------------------------------------------
#define S_IRWXU  0000700        // Owner user access permissions mask.
//
#define S_IRUSR  0000400        // Owner user read access permission.
#define S_IWUSR  0000200        // Owner user write access permission.
#define S_IXUSR  0000100        // Owner user execution or cross perm.
//----------------------------------------------------------------------
// Group owner access permissions.
//----------------------------------------------------------------------
#define S_IRWXG  0000070        // Owner group access permissions mask.
//
#define S_IRGRP  0000040        // Owner group read access permission.
#define S_IWGRP  0000020        // Owner group write access permission.
#define S_IXGRP  0000010        // Owner group execution or cross perm.
//----------------------------------------------------------------------
// Other users access permissions.
//----------------------------------------------------------------------
#define S_IRWXO  0000007        // Other users access permissions mask.
//
#define S_IROTH  0000004        // Other users read access permission.
#define S_IWOTH  0000002        // Other users write access permissions.
#define S_IXOTH  0000001        // Other users execution or cross perm.
//----------------------------------------------------------------------
// S-bit: in this case there is no mask to select all of them.
//----------------------------------------------------------------------
#define S_ISUID  0004000        // S-UID.
#define S_ISGID  0002000        // S-GID.
#define S_ISVTX  0001000        // Sticky.
//----------------------------------------------------------------------
// Macro-instructions to verify the type of file.
//----------------------------------------------------------------------
#define S_ISBLK(m)   (((m) & S_IFMT) == S_IFBLK)   // Block device.
#define S_ISCHR(m)   (((m) & S_IFMT) == S_IFCHR)   // Character device.
#define S_ISFIFO(m)  (((m) & S_IFMT) == S_IFIFO)   // FIFO.
#define S_ISREG(m)   (((m) & S_IFMT) == S_IFREG)   // Regular file.
#define S_ISDIR(m)   (((m) & S_IFMT) == S_IFDIR)   // Directory.
#define S_ISLNK(m)   (((m) & S_IFMT) == S_IFLNK)   // Symbolic link.
#define S_ISSOCK(m)  (((m) & S_IFMT) == S_IFSOCK)  // Socket.
//----------------------------------------------------------------------
// Structure `stat'.
//----------------------------------------------------------------------
struct stat {
    dev_t     st_dev;       // Device containing the file.
    ino_t     st_ino;       // File serial number (inode number).
    mode_t    st_mode;      // File type and permissions.
    nlink_t   st_nlink;     // Links to the file.
    uid_t     st_uid;       // Owner user id.
    gid_t     st_gid;       // Owner group id.
    dev_t     st_rdev;      // Device number if it is a device file.
    off_t     st_size;      // File size.
    time_t    st_atime;     // Last access time.
    time_t    st_mtime;     // Last modification time.
    time_t    st_ctime;     // Last inode modification.
    blksize_t st_blksize;   // Block size for I/O operations.
    blkcnt_t  st_blocks;    // File size / block size.
};
//----------------------------------------------------------------------
// Function prototypes.
//----------------------------------------------------------------------
int    chmod  (const char *path, mode_t mode);
int    fchmod (int fdn, mode_t mode);
int    fstat  (int fdn, struct stat *buffer);
int    lstat  (const char *restrict path, struct stat *restrict buffer);
int    mkdir  (const char *path, mode_t mode);
int    mkfifo (const char *path, mode_t mode);
int    mknod  (const char *path, mode_t mode, dev_t dev);
int    stat   (const char *restrict path, struct stat *restrict buffer);
mode_t umask  (mode_t mask);

#endif // _SYS_STAT_H
