#include <unistd.h>
#include <sys/types.h>
#include <sys/os32.h>
#include <errno.h>
//----------------------------------------------------------------------
char *optarg;
int   optind    = 1;
int   opterr    = 1;
int   optopt    = 0;
//----------------------------------------------------------------------
static void getopt_no_argument (int opt);
//----------------------------------------------------------------------
int
getopt (int argc, char *const argv[], const char *optstring)
{
    static int o = 0;           // Index to scan grouped options.
           int s;               // Index to scan `optstring'
           int opt;             // Current option letter.
           int flag_argument;   // If there should be an argument.
    //
    // Entering the function, `flag_argument' is zero. Just to make
    // it clear:
    //
    flag_argument = 0;
    //
    // Scan `argv[]' elements, starting form the value that `optind'
    // already have.
    //
    for (; optind < argc; optind++)
      {
        //
        // If an option is expected, some check must be done at
        // the beginning.
        //
        if (!flag_argument)
          {
            //
            // Check if the scan is finished and `optind' should be kept
            // untouched:
            //   `argv[optind]'    is a null pointer;
            //   `argv[optind][0]' is not the character `-';
            //   `argv[optind]'    points to the string "-";
            //   all `argv[]' elements are parsed.
            //
            if (argv[optind] == NULL
             || argv[optind][0] != '-'
             || argv[optind][1] == 0
             || optind >= argc)
              {
                return (-1);
              }
            //
            // Check if the scan is finished and `optind' is to be
            // incremented:
            //   `argv[optind]'    points to the string "--".
            //
            if (argv[optind][0] == '-'
             && argv[optind][1] == '-'
             && argv[optind][2] == 0)
              {
                optind++;
                return (-1);
              }
          }
        //
        // Scan `argv[optind]' using the static index `o'.
        //
        for (; o < strlen (argv[optind]); o++)
          {
            //
            // If there should be an option, index `o' should
            // start from 1, because `argv[optind][0]' must
            // be equal to '-'.
            //
            if (!flag_argument && (o == 0))
              {
                //
                // As there is no options, `o' cannot start
                // from zero, so a new loop is done.
                //
                continue;
              }
            //
            if (flag_argument)
              {
                //
                // There should be an argument, starting from
                // `argv[optind][o]'.
                //
                if ((o == 0) && (argv[optind][o] == '-'))
                  {
                    //
                    // `argv[optind][0]' is equal to '-', but there
                    // should be an argument instead: the argument
                    // is missing.
                    //
                    optarg = NULL;
                    //
                    if (optstring[0] == ':')
                      {
                        //
                        // As the option string starts with `:' the
                        // function must return `:'.
                        //
                        optopt = opt;
                        opt = ':';
                      }
                    else
                      {
                        //
                        // As the option string does not start with `:'
                        // the function must return `?'.
                        //
                        getopt_no_argument (opt);
                        optopt = opt;
                        opt = '?';
                      }
                    //
                    // `optind' is left untouched.
                    //
                  }
                else
                  {
                    //
                    // The argument is found: `optind' is to be
                    // incremented and `o' is reset.
                    //
                    optarg = &argv[optind][o];
                    optind++;
                    o = 0;
                  }
                //
                // Return the option, or ':', or '?'.
                //
                return (opt);
              }
            else
              {
                //
                // It should be an option: `optstring[]' must be
                // scanned.
                //
                opt = argv[optind][o];
                //
                for (s = 0, optopt = 0; s < strlen (optstring); s++)
                  {
                    //
                    // If `optsting[0]' is equal to ':', index `s' must
                    // start at 1.
                    //
                    if ((s == 0) && (optstring[0] == ':'))
                      {
                        continue;
                      }
                    //
                    if (opt == optstring[s])
                      {
                        //
                        if (optstring[s+1] == ':')
                          {
                            //
                            // There is an argument.
                            //
                            flag_argument = 1;
                            break;
                          }
                        else
                          {
                            //
                            // There is no argument.
                            //
                            o++;
                            return (opt);
                          }
                      }
                  }
                //
                if (s >= strlen (optstring))
                  {
                    //
                    // The `optstring' scan is concluded with no
                    // match.
                    //
                    o++;
                    optopt = opt;
                    return ('?');
                  }
                //
                // Otherwise the loop was broken.
                //
              }
          }
        //
        // Check index `o'.
        //
        if (o >= strlen (argv[optind]))
          {
            //
            // There are no more options or there is no argument
            // inside current `argv[optind]' string. Index `o' is
            // reset before the next loop.
            //
            o = 0;
          }
      }
    //
    // No more elements inside `argv' or loop broken: there might be a
    // missing argument.
    //
    if (flag_argument)
      {
        //
        // Missing option argument.
        //
        optarg = NULL;
        //
        if (optstring[0] == ':')
          {
            return (':');
          }
        else
          {
            getopt_no_argument (opt);
            return ('?');
          }
      }
    //
    return (-1);
}
//----------------------------------------------------------------------
static void
getopt_no_argument (int opt)
{
    if (opterr)
      {
        fprintf (stderr, "Missing argument for option `-%c'\n", opt);
      }
}
